use std::str::FromStr;

struct GlobalArgs {
    subcommand: Option<Subcommand>,
}

enum Subcommand {
    Repl,
    Help,
}

#[derive(Debug)]
pub enum ParseArgsError {
    InvalidSubcommand(String),
    UnexpectedArgument(String),
}
impl std::error::Error for ParseArgsError {}
impl std::fmt::Display for ParseArgsError {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            ParseArgsError::InvalidSubcommand(c) => write!(f, "invalid subcommand: {}", c),
            ParseArgsError::UnexpectedArgument(c) => write!(f, "unexpected argument: {}", c),
        }
    }
}

impl FromStr for Subcommand {
    type Err = ParseArgsError;

    fn from_str(input: &str) -> Result<Subcommand, Self::Err> {
        // TODO: errors should show similar commands?
        match input {
            "help" => Ok(Subcommand::Help),
            "repl" => Ok(Subcommand::Repl),
            _ => Err(ParseArgsError::InvalidSubcommand(input.to_string())),
        }
    }
}

fn usage(exit_code: i32) -> ! {
    println!("Hang in there! TODO: Implement help!");
    std::process::exit(exit_code);
}

fn parse_global_args() -> Result<GlobalArgs, ParseArgsError> {
    let mut subcommand = None;
    let mut args = std::env::args();

    while let Some(arg) = args.next() {
        match arg.as_str() {
            "--help" | "-h" => {
                usage(0);
            }
            s if subcommand.is_none() => {
                subcommand = Some(s.parse()?);
            }
            _ => return Err(ParseArgsError::UnexpectedArgument(arg.to_string())),
        }
    }

    return Ok(GlobalArgs { subcommand });
}

pub fn run() -> crate::Result<()> {
    Ok(())
}
